#!/usr/bin/perl

#perl ~oksana/PERL/Change_headders.pl <input fasta file> -acc <ACCESSION number for genome>


use strict;
use warnings;
use lib "/tools/lib/perl5/site_perl/5.12.2";
use Bio::Perl;
use Bio::SeqIO;
use Bio::SeqIO::fasta;
use Bio::SeqFeature::Generic;
use Getopt::Long;

my ($acc, $Help, $prodigal, $prodigal_runner, $input, $output);
&GetOptions (
	"i|input=s" => \$input,
	"o|output=s" => \$output, 
        "acc=s" => \$acc,   # defines accession number for the genome
        "prodigal" => \$prodigal, #if prodigal directly was ran on the assembled genome
        "prodigal_runner" => \ $prodigal_runner, #if prodigal_runner was ran on the assembled genome
        "h|help" => \$Help    # Prints the help text
);

####			  		 ####
#### Extracting the sequences to compare ####
####			 		 ####


### Extracting the genes from the File 1 ###
my $fasta_1 = Bio::SeqIO->new(-file => $input ,-format => 'Fasta');
my (%Satisfying_1, %Satisfying_2, %Both_Unique);
while ( my $seq_object = $fasta_1->next_seq() ) {
	my ($PROT_seq, $ID, $PROT_seq_length, $Description, $NewID)='';
	$PROT_seq = $seq_object->seq;
	$PROT_seq =~ s/\*// if $PROT_seq =~ m/\*$/;
	if (defined $prodigal) { 
		($ID) = ($seq_object->id() =~ /(.{0,16})/);
		if ($seq_object->description =~ m/#\s(\d+)\s#\s(\d+)\s#.+/) {
			$NewID = "CDS_".$1."-".$2;
		}
	} elsif (defined $prodigal_runner) {
		$NewID = $seq_object->id;
	}
	$Description = $input;
	$Description =~ s/_prodigal\.orf\.fsa//;
	#$Description =~ s/\..+\..*$//;
	#$PROT_seq_length = length($PROT_seq);
	$Satisfying_1{$NewID}->{'Description'}=$Description;
	$Satisfying_1{$NewID}->{'Seq'}=$PROT_seq;
}

####			  ####
#### Printing the results ####
####			  ####

open (OUT, '>', $output) or die "Can't read file $output: $!";
foreach my $key (sort (keys %Satisfying_1)) {
        my $seq = $Satisfying_1{$key}->{'Seq'};
	my $Description = $Satisfying_1{$key}->{'Description'};
	$Description =~ s/\_/ /g;
	my $Header;
	if (defined $acc) {
        	$Header = ">".$acc."_".$key." ".$Description;
	} else {
		$Header = ">".$key." ".$Description;
	}
        print OUT $Header."\n";
        for (my $i =0;$i<length($seq);$i+=60){
                print OUT substr($seq, $i,60)."\n";
        }       
}

