#!/usr/bin/perl


use strict;
use warnings;
use Bio::Perl;
use Bio::SeqIO::fasta;
use Bio::SeqFeature::Generic;
use Cwd;
use Cwd qw(chdir);
use lib qw(/Users/oksana/PhD/Scripts);
use lib qw(/Users/oksana/PhD/Scripts/go-perl);
use PostScript::Simple 0.07;
use File::Basename;
use Bio::SearchIO;
use Getopt::Long;
use File::Path;
use File::Glob;
use Data::Dumper;
use GO::Parser;


### STEPS OF ANALYSIS: ###
# 1. Define a set of genomes to analyse => -i, -c, -u, -ci options.
# 2. Output 


#############################################
########### OPTIONS, MEMORY, PATH ###########
#############################################

my ($keep, $Help, $allprofiles, $slack, $single, $uid,
 	$interpro_scans, $outfile, $pathways, $pathwayDB, $stats, $dir, $outPath);
my ($core, $pan, $specific, $i2c, $i2ci, $u2c, $u2ci, @Set1, @Set2, $set_choice, 
	$analysis, $analysistype, $profiles2output, $resultvisuals, $type, $R_version, $scripts_path);
my (@MEM_Pfam, @MEM_Profiles, @MEM_Superfamily);
&GetOptions (
	"keep:s" => \$keep, 
	"set1=s" => \@Set1, ## List of genomes in fasta format
	"set2=s" => \@Set2, ## List of genomes in fasta format
	
	"analysis=s" => \$analysis, #core, pan, pancoreplot, pancore_matrix, specific_matrix, specific
	"specific_type=s" => \$analysistype,
	"profilesoutput=s" => \$profiles2output, #single, multiple
	"resultvisuals=s" => \$resultvisuals, #pathways, statistics
	"type=s" => \$type, # assembly, genbank, proteins

	"root=s" => \$dir,
	"OutPath=s" => \$outPath,
	"scripts_path=s" => \$scripts_path,
	"R_version=s" => \$R_version,
	
	"h|help"=> \$Help 
);


print_help() if (defined $Help);

######################################################################################
############################### DATA PREPARATION #####################################
######################################################################################

#my $R = '/usr/cbs/tools/bin/linux64/R-2.12';
my $R = 'R';
$keep = 'PanFunPro' unless defined $keep;
$keep =~ s/\/$//;

my $cwd = getcwd();
$dir = $cwd unless defined $dir;
$dir =~ s/\/$//;

die("ERROR! You must specify a valid directory. Run PanFunPro.pl with '-keep' option.\n") unless (-r $dir);

@Set1 = PrepareSet(@Set1) if (scalar @Set1);
@Set2 = PrepareSet(@Set2) if (scalar @Set2); 
$pathwayDB = 'GO' unless defined $pathwayDB;
$profiles2output = 'single' unless defined $profiles2output;
$resultvisuals = 'none' unless defined $resultvisuals;
$slack = 0 unless defined $slack; 
$outPath = $dir."/Output" unless defined $outPath;
mkpath($outPath) unless -e $outPath;
$R_version = 3 unless defined $R_version;
$analysistype = 'i2c' unless defined $analysistype;
$scripts_path = '.' unless defined $scripts_path;
$scripts_path =~ s/\/$//;


my %Outhash;
###############################################################################################
##################################### RUNNING OPTIONS #########################################
###############################################################################################

if ($analysis eq 'core' or $analysis eq 'pan' or $analysis eq 'specific') {
	## Calculating the core, pan, and specific genomes
	if ($analysis eq 'core') {
		print STDERR "Calculating the intersection between genomes\n";
		%Outhash = IntersectionWithSlack(@Set1);
		print STDERR keys(%Outhash)." core gene families"."\n";
	} elsif ($analysis eq 'pan') {
		print STDERR "Calculating the pan between genomes\n";
		%Outhash = Union(@Set1);
		print STDERR keys(%Outhash)." pan gene families"."\n";
	} elsif ($analysis eq 'specific') {
		if ($analysistype eq 'i2c') {
			print STDERR "Calculating the core between one set of genomes which is complimentary to a pan of another set of genomes\n";
			my %Set1=IntersectionWithSlack(@Set1);
			my %Set2=Union(@Set2);
			%Outhash = Complementary(\%Set1, \%Set2);
			print STDERR keys(%Outhash)." specific gene families"."\n";
		} elsif ($analysistype eq 'u2c') {
			print STDERR "Calculating the pan between one set of genomes which is complimentary to a pan of another set of genomes\n";
			my %Set1= Union(@Set1);
			my %Set2 = Union(@Set2);
			%Outhash = Complementary(\%Set1, \%Set2);
			print STDERR keys(%Outhash)." specific gene families"."\n";
		} elsif ($analysistype eq 'i2ci') {
			print STDERR "Calculating the core between one set of genomes which is complimentary to a core of another set of genomes\n";
			my %Set1=IntersectionWithSlack(@Set1);
			my %Set2=IntersectionWithSlack(@Set2);
			%Outhash = Complementary(\%Set1, \%Set2);
			print STDERR keys(%Outhash)." specific gene families"."\n";
		} elsif ($analysistype eq 'u2ci') {
			print STDERR "Calculating the pan between one set of genomes which is complimentary to a core of another set of genomes\n";
			my %Set1=Union(@Set1);
			my %Set2=IntersectionWithSlack(@Set2);
			%Outhash = Complementary(\%Set1, \%Set2);
			print STDERR keys(%Outhash)." specific gene families"."\n";
		}
	} 
	
	## Printing out the FASTA file with sequences
	if ($profiles2output eq 'multiple') {
		my $profiles_out = $cwd."/PanFunPro.profiles.fsa";
		&Printing_seq_all(\%Outhash, $profiles_out);
		system "cp PanFunPro.profiles.fsa $outPath";
	} elsif ($profiles2output eq 'single') { 
		my $org_name;
		$org_name = PickRandomKey(%Outhash);
		my $profiles_out = $cwd."/PanFunPro.profiles.fsa";
		&Printing_seq_organism($org_name, \%Outhash, $profiles_out);
		system "cp PanFunPro.profiles.fsa $outPath";
	}
	
	
	## Additional figures with statistics and pathways anaysis of calculated core, pan, and specific genomes
	if ($resultvisuals eq 'Statistics') {
		my $lentbl = $outPath."/PanFunPro.lengthstable.txt";
		my $lenplot = $outPath."/PanFunPro.lengthsplot.ps";
		if ($R_version eq '2') {
			Distribution_R2(\%Outhash, $lentbl, $lenplot, $R);
		}elsif ($R_version eq '3') {
			Distribution_R3(\%Outhash, $lentbl, $lenplot, $R);
		}
	}
	if ($resultvisuals eq 'GO') {
		my $pathway2GOids = $cwd."/PanFunPro.GO_IDs.txt"; 
		my $profile2tbl = $cwd."/PanFunPro.profilelist.txt";
		my %Profiles2List = CountValuesPerKey(\%Outhash, $profile2tbl);
		my %Pathways2list = Pathways2list(\%Profiles2List);
		&PrintPWlist (\%Pathways2list, $pathway2GOids);
		my $pathway2GOterms = $cwd."/PanFunPro.GO2terms.txt";
		my $tmp_tbl = $cwd."/PanFunPro.GO2slim.tmp.txt";
		my %GOTerms = GO2Terms($pathway2GOids, $pathway2GOterms, $tmp_tbl, $R);
		my $GO2count = $outPath."/PanFunPro.GOTerms2Count.txt";
		my $GO2plot = $outPath."/PanFunPro.GOTerms2Count.ps";
		if ($R_version eq '2') {
			&GO2Term_plot_R2(\%GOTerms, $GO2count, $GO2plot, $R);
		} elsif ($R_version eq '3') {
			&GO2Term_plot_R3(\%GOTerms, $GO2count, $GO2plot, $R);
		}
	}
} elsif ($analysis eq 'pancoreplot') {
	my @GenomeList = SearchFiles(@Set1);
	my $plottbl = $outPath."/PanFunPro.PanCorePlotTable.txt";
	my $plotname = $outPath."/PanFunPro.PanCorePlot.ps";
	&PanCoreCalculation($plottbl, \@GenomeList);
	&pancoreplot($plottbl, $plotname, $R);
} elsif ($analysis eq 'pancore_matrix') {
	my @GenomeList = SearchFiles(@Set1);
	my $plottbl = $outPath."/PanFunPro.PanCoreMatrixTable.txt";
	my $plotname = $outPath."/PanFunPro.PanCoreMatrix.ps";
	&PanCoreMatrix($plottbl, $plotname, $plottbl, \@GenomeList);
} elsif ($analysis eq 'specific_matrix') {
	my @GenomeList = SearchFiles(@Set1);
	my $SpecificMatrixTBL = $outPath."/PanFunPro.SpecificMatrixTable.txt";
	my $SpecificMatrixPLOT = $outPath."/PanFunPro.SpecificMatrix.ps";
	if ($R_version eq '2') {
		SpecificMatrix_plot_R2(\@GenomeList, $SpecificMatrixTBL, $SpecificMatrixPLOT,$R);
	} elsif ($R_version eq '3') {
		SpecificMatrix_plot_R3(\@GenomeList, $SpecificMatrixTBL, $SpecificMatrixPLOT,$R);
	}
}

#####################################
#### FINDING CORE, PAN, SPECIFIC ####
#####################################

sub PrepareSet {
	@_ = split ",", join(",", @_);
	for (@_) {
		$_ = (/(\w+)[:\-]([0-9]+)/ ? join(",", ($1 .. $2)) : $_);
	}
	@_ = split ",", join(",", @_);
	return @_;
}

sub SearchFiles {
	my (@Genome_list, @Genome_list_tmp);
	foreach my $item (@_) {
		push(@Genome_list, glob("$cwd/$keep/*\.$item\.profiles"));
	}
	return @Genome_list;
}

sub SearchOrganism {
	my ($organism_num) = @_;
	my $OrganismFile = glob("$cwd/*.$organism_num.profiles");
	my $OrganismPID = $OrganismFile;
	return $OrganismPID;
}

sub ReadFile {
	my ($InFile) = @_ ;
	my %ProteinList;
	my $PID = $InFile;
	my $fasta = Bio::SeqIO->new(-file => $InFile ,-format => 'Fasta');
	while ( my $seq_object = $fasta->next_seq() ) {
        	my ($Seq, $ID, $Description, $ID_new) = '';
                $Seq = $seq_object->seq;
        	$ID = $seq_object->id;
		if ($ID =~ m/^(.+)\.\d+/) {
			$ID_new = $1;
		}
        $Description = $seq_object->description;
		my ($GeneID, $FunctionID, $DescriptionID, $Acc, $PathwayID);
		if ($Description =~ m/ID\=\"(.+)\"\sFunction\=\"(.*)\"\sDescription\=\"(.*)"\sPathway\=\"(.*)\"/) {
			$GeneID = $1;
			$FunctionID = $2;
			$DescriptionID = $3;
			$PathwayID = $4;
		} else {
			print STDERR "Doesn't match\n";
		}
		my $newGeneID = $PID."___".$GeneID;
		$ProteinList{$ID_new}->{'Genes'}->{$newGeneID}->{'Function'}=$FunctionID;
		$ProteinList{$ID_new}->{'Genes'}->{$newGeneID}->{'Pathway'}=$PathwayID;
		$ProteinList{$ID_new}->{'Genes'}->{$newGeneID}->{'PID'}=$PID;
		$ProteinList{$ID_new}->{'Genes'}->{$newGeneID}->{'Description'}=$DescriptionID;
		$ProteinList{$ID_new}->{'Genes'}->{$newGeneID}->{'Seq'}=$Seq;
		
		#push (@{$ProteinList{$ID_new}}, [$GeneID, $FunctionID, $DescriptionID, $Seq, $PID]);
	}	
	return %ProteinList;
}


sub CountKeys {
	my (%Genome) = @_;
	my $count = keys (%Genome);
	return $count;
} 		

sub Intersection {
	my @Genome_list = SearchFiles(@_);
	my (%Intersection) =();
	for (my $i=0; $i<=$#Genome_list; $i++) {
		my %TMP_Intersection;
		my %TMP_Profiles = ReadFile($Genome_list[$i]);
		if ($i == 0) {
			%Intersection = %TMP_Profiles;
		} else {
			foreach my $key (sort keys %TMP_Profiles) {
				if (exists $Intersection{$key}) {
					$TMP_Intersection{$key} = $Intersection{$key};
					foreach my $key1 (keys %{$TMP_Profiles{$key}->{'Genes'}}) {
						$TMP_Intersection{$key}->{'Genes'}->{$key1}->{'Function'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Function'};
						$TMP_Intersection{$key}->{'Genes'}->{$key1}->{'Pathway'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Pathway'};
						$TMP_Intersection{$key}->{'Genes'}->{$key1}->{'PID'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'PID'};
						$TMP_Intersection{$key}->{'Genes'}->{$key1}->{'Description'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Description'};
						$TMP_Intersection{$key}->{'Genes'}->{$key1}->{'Seq'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Seq'};
					}
				}
			}
			%Intersection = %TMP_Intersection;
			%TMP_Intersection = ();	
		}
	}
	return %Intersection;
}

sub Union {
	my @Genome_list = SearchFiles(@_);
	my (%Union) =();
	for (my $i=0; $i<=$#Genome_list;$i++) {
		my (%TMP_Union) = ();
		my %TMP_Profiles = ReadFile($Genome_list[$i]);
		if ($i == 0) {
			%Union = %TMP_Profiles;
		} else {
			foreach my $key (sort keys %TMP_Profiles) {
				foreach my $key1 (keys %{$TMP_Profiles{$key}->{'Genes'}}) {
					$Union{$key}->{'Genes'}->{$key1}->{'Function'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Function'};
					$Union{$key}->{'Genes'}->{$key1}->{'Pathway'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Pathway'};
					$Union{$key}->{'Genes'}->{$key1}->{'PID'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'PID'};
					$Union{$key}->{'Genes'}->{$key1}->{'Description'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Description'};
					$Union{$key}->{'Genes'}->{$key1}->{'Seq'}=$TMP_Profiles{$key}->{'Genes'}->{$key1}->{'Seq'};
					 
				}
			}
		}	
	}
	return %Union;
}

sub Complementary {
	my ($Hash1, $Hash2) = @_;
	my (%Complement)=();
	foreach my $key (keys %{$Hash1}) {
		if (!exists $Hash2->{$key}) {
			$Complement{$key}=$Hash1->{$key};
		} else {
		}
	} 
	return %Complement;
}

sub Printing_seq_all {
	my ($Hash, $out) = @_;
	unlink $out;
	open (OUT, '>>', $out);
	foreach my $item (sort keys %{$Hash}) {
		my $i = 0;
		foreach my $item1 (keys %{$Hash->{$item}->{'Genes'}}) {
			my @row = split('___', $item1);
			my $GeneID = $row[1];
			print OUT ">".$item.".$i"." ID=\"".$GeneID."\" Function=\"".$Hash->{$item}->{'Genes'}->{$item1}->{'Function'}.
			"\" Description=\"".$Hash->{$item}->{'Genes'}->{$item1}->{'Description'}."\" Pathway=\"".$Hash->{$item}->{'Genes'}->{$item1}->{'Pathway'}."\" PID=\"".
			$Hash->{$item}->{'Genes'}->{$item1}->{'PID'}."\"\n".$Hash->{$item}->{'Genes'}->{$item1}->{'Seq'}."\n";
			$i ++;
		}
	}
	close OUT;
}


sub IntersectionWithSlack {
	my @Genome_list = SearchFiles(@_);
	my %SlackUnion = Union(@_); 
	my (%SlackIntersection) =();
	my $num = scalar(@Genome_list) - $slack;
	foreach my $key (keys %SlackUnion) {
		my %VarCount = ();
		foreach my $key1 (keys %{$SlackUnion{$key}->{'Genes'}}) {
			my $PID = $SlackUnion{$key}->{'Genes'}->{$key1}->{'PID'};
			$VarCount{$PID}=$key;
		}
		my $GenomeCount = CountKeys(%VarCount);
		if ($GenomeCount >= $num) {
			$SlackIntersection{$key}=$SlackUnion{$key};
		}
	}
	return %SlackIntersection;
}

sub PickRandomKey {	
	my (%Hash) = @_;
	my @Array;
	foreach my $item (keys %Hash) {
		@Array = keys(%{$Hash{$item}->{'Genes'}});
	}
	my $random_key = '';
	$random_key = $Array[int(rand(@Array))];
	my $random_num = (split /___/, $random_key)[0];
	return $random_num;
}

sub Printing_seq_organism {
	my ($OrgDescr, $Hash, $out) = @_;
	unlink $out;
	open (OUT, '>>', $out);
	foreach my $item (keys %{$Hash}) {
		my $i = 0;
		foreach my $item1 (keys %{$Hash->{$item}->{'Genes'}}) {
			my $gene_id = (split /___/, $item1)[1];
			if ($Hash->{$item}->{'Genes'}->{$item1}->{'PID'} eq $OrgDescr) {
				print OUT ">".$item.".$i"." ID=\"".$gene_id."\" Function=\"".$Hash->{$item}->{'Genes'}->{$item1}->{'Function'}.
				"\" Description=\"".$Hash->{$item}->{'Genes'}->{$item1}->{'Description'}."\" Pathway=\"".$Hash->{$item}->{'Genes'}->{$item1}->{'Pathway'}.
				"\" PID=\"".$Hash->{$item}->{'Genes'}->{$item1}->{'PID'}."\"\n".$Hash->{$item}->{'Genes'}->{$item1}->{'Seq'}."\n";
				$i ++;
			}
		}
	}
	close OUT;
}


#############################################
#### PanFunPro PAN-/CORE- PLOT OF SUBSET #### 
#############################################

sub OpenProfiles_PW {
	my %ProfileList;
	my $fasta = Bio::SeqIO->new(-file => @_ ,-format => 'Fasta');
	while ( my $seq_object = $fasta->next_seq() ) {
 		my $Seq = $seq_object->seq;
		my $ID = $seq_object->id;
		$ID =~ m/(.+)\.(\d+)$/;
		my $newID = $1;
		my $Description = $seq_object->description;
		my ($GeneID, $FunctionID, $DescriptionID, $PathwayID)='';
		if ($Description =~ m/ID\=\"(.+)\"\sFunction\=\"(.+)\"\sDescription\=\"(.+)\"\sPathway\=\"(.+)\"/) {
			$GeneID = $1;
			$FunctionID = $2;
			$DescriptionID = $3;
			$PathwayID = $4; 
		}
		$ProfileList{$newID}->{'Genes'}->{$GeneID}->{'Seq'}=$Seq;
		$ProfileList{$newID}->{'Genes'}->{$GeneID}->{'Description'}=$DescriptionID;
		$ProfileList{$newID}->{'Function'}=$FunctionID;
		$ProfileList{$newID}->{'Pathway'}=$PathwayID;
	}
	return %ProfileList;	
}

sub CountGenes {
	my ($ProfList) = @_;
	my $Totalnum = 0;
	foreach my $key (keys %{$ProfList}) {
		$Totalnum += keys( %{$ProfList->{$key}->{'Genes'}});
	}
	return $Totalnum;
}

sub PanCoreCalculation {
	my ($tbl_out, $List_in) = @_;
	unlink $tbl_out;
	my (%GenomeProfiles, %CoreGenome, %PanGenome, %NewGenes, $TotalGenes);
	open (TBLOUT, ">", $tbl_out) or die "Couldn't open $tbl_out";
	print TBLOUT "Id\tGenome Name\tTotal Genes\tGenome Profiles\tCore genome\tPan Genome\tNew Profiles\n";
	my $num = 0;
	foreach my $file (@{$List_in}) { 
		%GenomeProfiles = OpenProfiles_PW($file);
		$TotalGenes = CountGenes(\%GenomeProfiles);
		$num ++;
		if ($num == 1){
			%CoreGenome = %GenomeProfiles;
			%PanGenome = %GenomeProfiles;
			%NewGenes = %GenomeProfiles;
		} elsif ($num > 1) {
			my %TMP_CoreGenome = ();
			%NewGenes=();
			my ($i, $j)=0;
			foreach my $key (keys %GenomeProfiles) {
				if (exists $CoreGenome{$key}) {
					$i++;
					foreach my $key1 (keys %{$GenomeProfiles{$key}->{'Genes'}}) {
						if (!exists $CoreGenome{$key}->{'Genes'}->{$key1}) {
							$TMP_CoreGenome{$key}->{'Genes'}->{$key1}->{'Seq'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Seq'};
							$TMP_CoreGenome{$key}->{'Genes'}->{$key1}->{'Description'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Description'};
						}
					}
				} elsif (!exists $CoreGenome{$key})  {
					$NewGenes{$key}->{'Genes'}=$GenomeProfiles{$key}->{'Genes'};
				}
				foreach my $key1 (keys %{$GenomeProfiles{$key}->{'Genes'}}) {
					if (!exists $PanGenome{$key}->{'Genes'}->{$key1}) {
						$PanGenome{$key}->{'Genes'}->{$key1}->{'Seq'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Seq'};
						$PanGenome{$key}->{'Genes'}->{$key1}->{'Description'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Description'};
					}
				}
			}
			%CoreGenome=%TMP_CoreGenome;
			
		}
		my $num_id = $num-1;
		my ($f, $d) = fileparse($file);
		$f =~ s/\.profiles//;
		print TBLOUT $num_id."\t".$f."\t".$TotalGenes."\t".keys(%GenomeProfiles)."\t".keys(%CoreGenome)."\t".keys(%PanGenome)."\t".keys(%NewGenes)."\n";
	}
}

sub pancoreplot {
	my ($tbl, $panplot, $R_path) = @_;
	open R , "| $R_path --vanilla > /dev/null";
	print R "
postscript('$panplot');
layout( matrix(c(1,2), 1, 2, byrow = TRUE),widths=c(0.7,0.3))
data <- read.table('$tbl',skip=1,sep='\t',dec='.',header=FALSE);
tN <- data[,3]
x<-rbind(data[,4],data[,7])
r <- barplot(x,beside=TRUE,cex.names=0.7,names.arg=as.vector(data[,1])+1,ylim=c(0,1.3*max(data[,6])))
lines(r[2,], type='b', data[,5], col='red', lwd=3, cex=1)
lines(r[2,], type='b', data[,6], col='blue', lwd=3, cex=1)
legend(1,1.3*max(data[,6]),c('Total gene families','New gene families','Core genome','Pan genome'),col=c('darkgray','lightgray','red','blue'), lwd=c(4,4,4,4))
plot.new()
plot.window(xlim=c(0,2),ylim=c(-nrow(data),1))
for (i in 1:nrow(data)) {
 text(0,-i, adj=0,paste(i,': ',data[i,2]),cex=0.7)
}
dev.off();
";
	close R;
}

####################################
#### PanFunPro MATRIX OF SUBSET #### 
####################################

sub PanCoreMatrix {
	my ($tbl, $mat, $mattbl, $Array) = @_;
	my @LOF = @{$Array};
	my (%CMP, %displayMin, %displayMax);
	$displayMax{"paralogs"}=0;
	$displayMax{"orthologs"}=0;
	$displayMin{"paralogs"}=1;
	$displayMin{"orthologs"}=1;
	foreach (my $i=0;$i<=$#LOF;$i++) { 
		my $genomeprofiles = $LOF[$i];
		my ($f, $d) = fileparse($genomeprofiles);
		$f =~ s/\.profiles//;
		my %QueryProfiles = OpenProfiles_PW($genomeprofiles);
		my $QueryGenes =CountGenes(\%QueryProfiles);
		my $num1 = $i;
		$CMP{$num1}->{'Profiles'}=keys(%QueryProfiles);
		$CMP{$num1}->{'Name'}=$f;
		$CMP{$num1}->{'Genes'}=$QueryGenes;
		foreach (my $j=0;$j<=$#LOF;$j++) { 
			my $num2 = $j;
			my ($f1, $d1) = fileparse($LOF[$j]);
			$f1 =~ s/\.profiles//;
			print STDERR "PAIR: ".$num1." - ".$num2."\n";
			if ($num1 eq $num2) {
				my $common = WithinGenome(\%QueryProfiles);
				my $within = $common/keys(%QueryProfiles);
				my $within_rn = sprintf("%.1f", $within*100);
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'}=$within;
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Total'}=$CMP{$num1}->{'Profiles'};
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Name'}=$f1;
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Common'}=$common;
				$CMP{$num1}->{'Genomes'}->{$num2}->{'FractionText'}=$within_rn;
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Type'}='paralogs';
				if ($within < $displayMin{"paralogs"}) {
					$displayMin{"paralogs"} = $within;
				}
				if ($within > $displayMax{"paralogs"}) {
					$displayMax{"paralogs"} = $within;
				}
			} else {
				if (exists $CMP{$num2}->{'Genomes'}->{$num1}) {
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Total'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'Total'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Name'}=$f1;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Common'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'Common'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'Fraction'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'FractionText'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'FractionText'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Type'}='orthologs';
					if ($CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'} < $displayMin{"orthologs"}) {
						$displayMin{"orthologs"} = $CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'};
					}
					if ($CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'} > $displayMax{"orthologs"}) {
						$displayMax{"orthologs"} = $CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'};
					}
				} else {
					my $file1 = $LOF[$j];
					my ($f2, $d2) = fileparse(${LOF}[$j]);
					$f2 =~ s/\.profiles//;
					my $SubjectProfiles = $LOF[$j];
					my %SubjectProfiles = OpenProfiles_PW($SubjectProfiles);
					my $SubjectGenes = CountGenes(\%SubjectProfiles);
					my @Res_cmp = ComparePair(\%QueryProfiles, \%SubjectProfiles);
					my $core = $Res_cmp[0];
					my $pan = $Res_cmp[1];
					my $cmp = $core/$pan;
					my $cmp_rn = sprintf("%.1f", $cmp*100);
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Total'}=$pan;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Name'}=$f2;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Common'}=$core;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'}=$cmp;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'FractionText'}=$cmp_rn;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Type'}='orthologs';
					if ($cmp < $displayMin{"orthologs"}) {
						$displayMin{"orthologs"} = $cmp;
					}
					if ($cmp > $displayMax{"orthologs"}) {
						$displayMax{"orthologs"} = $cmp;
					}
				}
			}
		}
	}
	&PrintPS(\%CMP, \%displayMin, \%displayMax, $mat, $mattbl);
	&PrintMatrix(\%CMP, $mattbl);
}


sub WithinGenome {
	my ($Profiles)=@_;
	my $common;
	foreach my $key (keys %{$Profiles}) {
		if (keys (%{$Profiles->{$key}->{'Genes'}}) > 1) {
			$common++;
		}
	}
	return $common;
}

sub ComparePair {
	my ($Profiles1, $Profiles2)=@_;
	my @results;
	my %Total = (%{$Profiles1}, %{$Profiles2});
	my %Common = Core(\%{$Profiles1}, \%{$Profiles2});
	my $total = keys(%Total);
	my $common = keys(%Common);
	push (@results, $common, $total);
	return @results;
}

sub Core {
	my ($Profiles1, $Profiles2) = @_;
	my %Common;
	foreach my $key (keys %{$Profiles2}) {
		if (exists $Profiles1->{$key}) {
			$Common{$key}=$Profiles1;
		}
	}
	return %Common;
}

sub PrintMatrix {
	my ($Results, $fileout) = @_;
	open (OUT, '>', $fileout) or die "Can't read file $fileout: $!";
	print OUT "GENOMES";
;	foreach my $key (sort {$a <=> $b} keys %{$Results}) {
		print OUT "\t".$Results->{$key}->{'Name'};
	}
	print OUT "\n";
	foreach my $key (sort {$a <=> $b} keys %{$Results}) {
		print OUT $Results->{$key}->{'Name'};
		my @line = ();
		for my $item (0 .. keys (%{$Results->{$key}->{'Genomes'}})) {
			push (@line, $Results->{$key}->{'Genomes'}->{$item}->{'FractionText'});
		}
		my $line = join ("\t", @line);
		print OUT "\t".$line."\n";
	}
}


sub PrintPS {
	my ($CMP, $displayMn, $displayMx, $name) = @_;
	my $large = 1; # used with large number of proteomes (~50)

	my $DIM = keys(%{$CMP});
	# figure out if groups are being drawn
	my $draw_groups;
	
	my $horzVertTextdim = 40 ; # space assigned for text ADJUST HERE!!!!!
	$horzVertTextdim = 120 if defined $draw_groups;

	my $vertTitleSpace = 30;
	my $legendHeight = 6;
	$legendHeight = 12 if $large;
	my $legendWidth = 80;
	$legendWidth = 160 if $large;
	
	my $legendSteps = 20;
	my $legendMatrixDistance = -20;
	my @LEFT_RIGHT_MARGIN = (10,10);
	my @TOP_BOTTOM_MARGIN = (10,10);
	my $BOX_SIZE = 20;

	# obscure!!!!
	my $R = 1.6;
	my $R2 = 3;
	my $V2 = 0.5 * sqrt ( ($BOX_SIZE-$R)**2 + ($BOX_SIZE-$R)**2 );
	my $V5 = 0.5 * sqrt ( ($BOX_SIZE-$R2)**2 + ($BOX_SIZE-$R2)**2 );
	my $V = 0.5 * sqrt ( $BOX_SIZE**2 + $BOX_SIZE**2 );
	my $V3 = sqrt ( $R**2 + $R**2 );
	my $V4 = sqrt ( $R2**2 / 2 ) * 1.1;
	my $legendDist = 15;
	# paper size
	my $XSIZE = $legendMatrixDistance + ($legendDist + $legendWidth)  + $horzVertTextdim + sqrt ( $BOX_SIZE**2 + $BOX_SIZE**2 ) * $DIM + $LEFT_RIGHT_MARGIN[0] + $LEFT_RIGHT_MARGIN[1];
	my $YSIZE = $horzVertTextdim + $V * ($DIM + 0.5 ) + $TOP_BOTTOM_MARGIN[0] + $TOP_BOTTOM_MARGIN[1] + $vertTitleSpace + $legendHeight; 

	my @TOP = 
		($LEFT_RIGHT_MARGIN[0]+$horzVertTextdim + 0.5 * sqrt ( $BOX_SIZE**2 + $BOX_SIZE**2 ) * $DIM , 
		$YSIZE - $TOP_BOTTOM_MARGIN[0] - $horzVertTextdim
	);

	my $p = new PostScript::Simple(
	 xsize => $XSIZE,
	 ysize => $YSIZE,
	 colour => 1,
	 eps => 1,
	 units => "mm"
	);
	my $grp_dist = 70;
	my $last_group;
	$p->setfont("Palatino-Italic", 10);
	foreach my $s ( 1 .. $DIM  ) {
		foreach my $q ( $s .. $DIM  ) {
			my @top = ($TOP[0] + ($DIM-$q)*$V - ($s-1) * $V  , $TOP[1] - ($s-1) * $V - ($DIM - $q) * $V);
			my $type = "orthologs";
			$type = "paralogs" if $s == $q;
			my $fraction = $CMP->{$s-1}->{'Genomes'}->{$q-1}->{'Fraction'};
			$p->setcolour ((score2RGB($type,fraction2score($type,$fraction, \%{$displayMx}, \%{$displayMn}))));
			my @BOX;
			if ( $s == $q ) {
				@BOX = ( $top[0] , $top[1] , $top[0] - $V2, $top[1]-$V2 , $top[0] , $top[1]-2*$V2 , $top[0]+$V2 , $top[1]-$V2);
				$p->polygon({filled=>1},@BOX);
				my @BOX = (
					$top[0] - $V          , $top[1] - $V , 
					$top[0] - $V + $V4    , $top[1] - $V + $V4 , 
					$top[0]               , $top[1] - 2 * $V + 2 * $V4 ,
					$top[0] - $V4         , $top[1] - 2 * $V + $V4
				);
				$p->polygon({filled=>1},@BOX);
				@BOX = (
					$top[0] + $V          , $top[1] - $V , 
					$top[0] + $V - $V4    , $top[1] - $V + $V4 , 
					$top[0]               , $top[1] - 2 * $V + 2 * $V4 ,
					$top[0] + $V4         , $top[1] - 2 * $V + $V4 
			);
				$p->polygon({filled=>1},@BOX);
				$p->circle( {filled=>1} ,  $top[0] , $top[1]-2*$V5, $R2 );
			} else {
				@BOX = ( $top[0] , $top[1] , $top[0] - $V , $top[1]-$V , $top[0] , $top[1]-2*$V , $top[0]+$V , $top[1]-$V);
				$p->polygon({filled=>1},@BOX);
			}
			$p->setcolour ( 0,0,0);
			$p->setfont("Palatino-Bolditalic", 10);
			$p->text({rotate=>'0',align=>'center'},($top[0],$top[1]-$V+4), ,$CMP->{$q-1}->{'Genomes'}->{$s-1}->{'FractionText'}." %" );
			$p->setfont("Palatino-Italic", 10);
			my $sharedfamilies = $CMP->{$q-1}->{'Genomes'}->{$s-1}->{'Common'};
			$sharedfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
			my $totalfamilies = $CMP->{$q-1}->{'Genomes'}->{$s-1}->{'Total'};
			$totalfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
			$p->text({rotate=>'0',align=>'center'},($top[0],$top[1]-$V-1), ,"$sharedfamilies / $totalfamilies" );
		}
	}
	foreach my $s ( 1 .. $DIM  ) {
		foreach my $q ( $s .. $DIM  ) {
			my @top = ($TOP[0] - ($DIM-$q) * $V + ($s-1) * $V  , $TOP[1] - ($s-1) * $V - ($DIM - $q) * $V);
			$p->setcolour ( 255,255,255);
			$p->circle( {filled=>1} , $top[0] , $top[1] , $R );
		}
	}
	my %GROUPS;
	foreach my $q ( 1 .. $DIM  ) {
		$p->setcolour ( 0,0,0);
		$p->setfont("Palatino-Italic", 18);
		$p->text({rotate=>'-45',align=>'right'},($TOP[0] - ($q-1) * $V - 0.7*$V, $TOP[1] - ($q-1) * $V), ,$CMP->{$q-1}->{'Name'} );
		$p->setfont("Palatino-Roman", 10);
		$p->text({rotate=>'-45',align=>'right'},($TOP[0] - ($q-1) * $V - 0.5*$V - 5 - 1 , $TOP[1] - ($q-1) * $V - 5 + 1), ,$CMP->{$q-1}->{'Name'} );
		my $genes = $CMP->{$q-1}->{'Genes'};
		$genes =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		my $totalfamilies = $CMP->{$q-1}->{'Genomes'}->{$q-1}->{'Total'};
		$totalfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		$p->text({rotate=>'-45',align=>'right'},($TOP[0] - ($q-1) * $V - 0.5*$V - 8 - 1, $TOP[1] - ($q-1) * $V - 8 + 1 ), ,"$genes proteins, $totalfamilies families" );
	}
	my $Q = 0; # used to position titles and color boxes
	foreach my $s ( 1 .. $DIM  ) {
		$p->setcolour ( 0 , 0 , 0 );
		$p->setfont("Palatino-Italic", 18);
		$p->text({rotate=>'45',align=>'left'},($TOP[0] + ($DIM-$s) * $V + 0.7*$V,$TOP[1] - ($DIM-$s) * $V), ,$CMP->{$s-1}->{'Name'} );
		$p->setfont("Palatino-Roman", 10);
		$p->text({rotate=>'45',align=>'left'},($TOP[0] + ($DIM-$s) * $V + 0.5*$V + 5 + 1  ,$TOP[1] - ($DIM-$s) * $V - 5  + 1 ), ,$CMP->{$s-1}->{'Name'} );
		my $genes = $CMP->{$s-1}->{'Genes'};
		$genes =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		my $totalfamilies = $CMP->{$s-1}->{'Genomes'}->{$s-1}->{'Total'};
		$totalfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		$p->text({rotate=>'45',align=>'left'},($TOP[0] + ($DIM-$s) * $V + 0.5*$V + 8 + 1 ,$TOP[1] - ($DIM-$s) * $V - 8 + 1), ,"$genes proteins, $totalfamilies families" );
	}

	my $inc = 1/$legendSteps;
	my $legendBoxWidth = $legendWidth / $legendSteps;
	my $legendYpos_paralogs = $TOP_BOTTOM_MARGIN[1]+8;
	my $legendYpos_orthlogs = $YSIZE - $TOP_BOTTOM_MARGIN[0] - 12 - $legendHeight;
	my $legendXpos = $XSIZE - $LEFT_RIGHT_MARGIN[1] - $legendWidth - $legendDist;
	$p->setcolour ( 0 , 0 , 0 );

	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;

	$p->text({rotate=>'0',align=>'center'}, $legendXpos + $legendWidth / 2 , $legendYpos_paralogs+$legendHeight+2 , ,"Homology within proteomes");
	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;
	$p->text({rotate=>'0',align=>'center'},( $legendXpos + $legendWidth), $legendYpos_paralogs-9 , ,sprintf('%0.1f',(100*$displayMx->{"paralogs"})).' %');
	$p->text({rotate=>'0',align=>'center'}, $legendXpos, $legendYpos_paralogs-9 , ,sprintf('%0.1f',(100*$displayMn->{"paralogs"})).' %');
	for (my $i = 0 ; $i <= 1 ; $i+=$inc) {
		$p->setcolour ((score2RGB ("paralogs",$i)));
		my @BOX = (
			$legendXpos + (($i ) * $legendWidth) , $legendYpos_paralogs , 
			$legendXpos + (($i ) * $legendWidth), $legendYpos_paralogs+$legendHeight , 
			$legendXpos + (($i + $inc )* $legendWidth), $legendYpos_paralogs+$legendHeight , 
			$legendXpos + (($i + $inc ) * $legendWidth), $legendYpos_paralogs 
		);
		$p->polygon({filled=>1},@BOX);
	}
	$p->setcolour ( 0,0,0);
	$p->setfont("Palatino-Bold", 22);
	$p->setfont("Palatino-Roman", 18);
	$p->setcolour ( 0,0,0);
	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;
	$p->text({rotate=>'0',align=>'center'},$legendXpos + $legendWidth / 2 , $legendYpos_orthlogs+$legendHeight+2 , ,"Homology between proteomes");
	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;
	$p->text({rotate=>'0',align=>'center'},($legendXpos + $legendWidth), $legendYpos_orthlogs-9 , ,sprintf('%0.1f',100*$displayMx->{"orthologs"}).' %');
	$p->text({rotate=>'0',align=>'center'},$legendXpos, $legendYpos_orthlogs-9 , ,sprintf('%0.1f',100*$displayMn->{"orthologs"}).' %');
	for (my $i = 0 ; $i <= 1 ; $i+=$inc) {
		$p->setcolour ( (  score2RGB("orthologs",$i)));
		my @BOX = (
			$legendXpos + (($i ) * $legendWidth) , $legendYpos_orthlogs , 
			$legendXpos + (($i ) * $legendWidth), $legendYpos_orthlogs+$legendHeight , 
			$legendXpos + (($i + $inc )* $legendWidth), $legendYpos_orthlogs+$legendHeight , 
			$legendXpos + (($i + $inc ) * $legendWidth), $legendYpos_orthlogs 
		);
		$p->polygon({filled=>1},@BOX);
	}
	$p->setcolour ( 200,200,200);
	$p->setlinewidth(1.5);
	$p->line ( 
	$XSIZE-$LEFT_RIGHT_MARGIN[1] - 4  , $YSIZE-$TOP_BOTTOM_MARGIN[0] , 
	$XSIZE-$LEFT_RIGHT_MARGIN[1]  ,     $YSIZE-$TOP_BOTTOM_MARGIN[0] 
	);
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] , $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM + 1 
	);
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] -12  , $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM + 1 
	);
	$p->line ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] -12  ,  $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM - 1, 
		$XSIZE-$LEFT_RIGHT_MARGIN[1]  ,  $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM - 1 
	);
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1]  , $TOP_BOTTOM_MARGIN[1]
	) ;
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] -4 , $TOP_BOTTOM_MARGIN[1]
	) ;
	$p->output($name);
}

sub fraction2score {
	my ($type_in , $fraction_in, $displayMx, $displayMn) = @_;
	my $slope = 1/($displayMx->{$type_in} - $displayMn->{$type_in});
	my $intercept = 1 - ($slope * $displayMx->{$type_in});
	my $score = ($fraction_in*$slope) + $intercept;
	return $score;
}

sub score2RGB {
	# for instance 'paralogs'
	my ( $type_in , $score) = @_;
	my %Colours = &Colours;
	$score = 0 if $score < 0;
	$score = 1 if $score > 1;
	my @startColors = ($Colours{$type_in}->{'from_color'}->{'r'},$Colours{$type_in}->{'from_color'}->{'g'},$Colours{$type_in}->{'from_color'}->{'b'});
	my @endColors = ($Colours{$type_in}->{'to_color'}->{'r'},$Colours{$type_in}->{'to_color'}->{'g'},$Colours{$type_in}->{'to_color'}->{'b'});
	my @resultColor;
	foreach my $x (0 .. 2) {
		$resultColor[$x] = sprintf ( "%0.1f" ,( $startColors[$x] + ($endColors[$x] - $startColors[$x]) * $score) * 256);
	}
	return @resultColor;
}

sub Colours {
	my %Col;
	$Col{'orthologs'}->{'range_from'}='auto';
	$Col{"orthologs"}->{'range_to'}='auto';
	$Col{"orthologs"}->{'to_color'}->{'r'}=0;
	$Col{"orthologs"}->{'to_color'}->{'b'}=0.975;
	$Col{"orthologs"}->{'to_color'}->{'g'}=0.8;
	$Col{"orthologs"}->{'from_color'}->{'r'}=0.9;
	$Col{"orthologs"}->{'from_color'}->{'b'}=0.9;
	$Col{"orthologs"}->{'from_color'}->{'g'}=0.9;
	$Col{"paralogs"}->{'range_from'}='auto';
	$Col{"paralogs"}->{'range_to'}='auto';
	$Col{"paralogs"}->{'to_color'}->{'r'}=0.975;
	$Col{"paralogs"}->{'to_color'}->{'b'}=0;
	$Col{"paralogs"}->{'to_color'}->{'g'}=0;
	$Col{"paralogs"}->{'from_color'}->{'r'}=0.9;
	$Col{"paralogs"}->{'from_color'}->{'b'}=0.9;
	$Col{"paralogs"}->{'from_color'}->{'g'}=0.9;
	return %Col;
}

####################################
#### BASIC STATISTICS OF SUBSET #### 
####################################

# 1. Count Length Distribution
# 2. Count profiles: PF, CL, SSF, TIGR
sub Distribution_R2 {
	my ($InHash, $outtbl, $plot, $R_path) = @_;
	unlink $outtbl;
	open (OUT, '>>', $outtbl);
	print OUT "Type\tProfile\tAverageLength\n";
	foreach my $key (keys %{$InHash}) {
		my ($TotalLen, $Average) = 0;
		foreach my $key1 (keys %{$InHash->{$key}->{'Genes'}}) {
			my $len = length($InHash->{$key}->{'Genes'}->{$key1}->{'Seq'})*3;
			$InHash->{$key}->{'Genes'}->{$key1}->{'SeqLen'}=$len;
			$TotalLen += $len;
		}
		$Average = $TotalLen/keys(%{$InHash->{$key}->{'Genes'}});
		$InHash->{$key}->{'AverageGeneLength'}=$Average;
		if ($key =~ /^PF/) {print OUT "PfamA\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
		elsif ($key =~ /^SSF/) {print OUT "Superfamily\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
		elsif ($key =~ /^TIGR/) {print OUT "TIGRFAM\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
		elsif ($key =~ /^CL/) {print OUT "CD-HIT\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
	}
	open R , "| $R_path --vanilla > /dev/null";
	print R "
postscript('$plot')
library(ggplot2)
data <- read.table('$outtbl', header=T, sep='\t');
p_data <- c(sum(data\$Type == 'PfamA'), sum(data\$Type == 'TIGRFAM'),sum(data\$Type == 'Superfamily'),sum(data\$Type == 'CD-HIT'))
pie_df = data.frame(count = p_data, Type = c('PfamA', 'TIGRFAM', 'Superfamily', 'CD-HIT'))
pie_df\$fraction = pie_df\$count / sum(pie_df\$count)
pie_df\$fraction = pie_df\$fraction*100
pie_df = pie_df[order(pie_df\$fraction), ]
p1 = ggplot(data=pie_df, aes(x=Type, y=fraction, fill=Type))+geom_bar(stat='identity')+xlab('')+ylab('Fraction of genes (%)')+geom_text(aes(label=count), colour='black', size=3, vjust=1,hjust=1.1)+opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(),legend.position='none',panel.background=theme_rect(fill='transparent'))+scale_fill_brewer(palette='Set1')+scale_fill_discrete(name='')+coord_flip()+opts(axis.title.x = theme_text(size=11, angle=00, hjust=0.75, vjust=0),axis.text.y=theme_text(colour='black'))
data.data = data.frame(data\$Profile, data\$AverageLength, data\$Type);
p2 <- ggplot(data.data, aes(x=data.Type, y=data.AverageLength, fill=data.Type))+geom_boxplot(stat='boxplot')+xlab('')+ylab('Gene length (bp)')+scale_fill_brewer(palette='Set1')+opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(),legend.position='none',panel.background=theme_rect(fill='transparent'))+scale_fill_discrete(name='')+opts(axis.title.y = theme_text(size=12, angle=90, hjust=0.5, vjust=0.25),axis.text.x=theme_text(colour='black'))
vplayout <- function(x, y) viewport(layout.pos.row = x, layout.pos.col = y)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3, 2)))
print(p1,vp = vplayout(2, 1))
print(p2,vp = vplayout(1:3, 2))
dev.off();
";
	close R;
}

sub Distribution_R3 {
	my ($InHash, $outtbl, $plot, $R_path) = @_;
	unlink $outtbl;
	open (OUT, '>>', $outtbl);
	print OUT "Type\tProfile\tAverageLength\n";
	foreach my $key (keys %{$InHash}) {
		my ($TotalLen, $Average) = 0;
		foreach my $key1 (keys %{$InHash->{$key}->{'Genes'}}) {
			my $len = length($InHash->{$key}->{'Genes'}->{$key1}->{'Seq'})*3;
			$InHash->{$key}->{'Genes'}->{$key1}->{'SeqLen'}=$len;
			$TotalLen += $len;
		}
		$Average = $TotalLen/keys(%{$InHash->{$key}->{'Genes'}});
		$InHash->{$key}->{'AverageGeneLength'}=$Average;
		if ($key =~ /^PF/) {print OUT "PfamA\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
		elsif ($key =~ /^SSF/) {print OUT "Superfamily\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
		elsif ($key =~ /^TIGR/) {print OUT "TIGRFAM\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
		elsif ($key =~ /^CL/) {print OUT "CD-HIT\t".$key."\t".$InHash->{$key}->{'AverageGeneLength'}."\n";}
	}
	open R , "| $R_path --vanilla > /dev/null";
	print R "
postscript('$plot')
library(ggplot2)
library(grid)
data <- read.table('$outtbl', header=T, sep='\t');
p_data <- c(sum(data\$Type == 'PfamA'), sum(data\$Type == 'TIGRFAM'),sum(data\$Type == 'Superfamily'),sum(data\$Type == 'CD-HIT'))
pie_df = data.frame(count = p_data, Type = c('PfamA', 'TIGRFAM', 'Superfamily', 'CD-HIT'))
pie_df\$fraction = pie_df\$count / sum(pie_df\$count)
pie_df\$fraction = pie_df\$fraction*100
pie_df = pie_df[order(pie_df\$fraction), ]
p1 = ggplot(data=pie_df, aes(x=Type, y=fraction, fill=Type))+geom_bar(stat='identity')+xlab('')+ylab('Fraction of genes (%)')+geom_text(aes(label=count), colour='black', size=3, vjust=1,hjust=1.1)+theme(panel.grid.minor=element_blank(), panel.grid.major=element_blank(),legend.position='none')+scale_fill_brewer(palette='Set2')+coord_flip()+theme(axis.title.x = element_text(size=11, angle=00, hjust=0.75, vjust=0),axis.text.y=element_text(colour='black'))
data.data = data.frame(data\$Profile, data\$AverageLength, data\$Type);
p2 <- ggplot(data.data, aes(x=data.Type, y=data.AverageLength, fill=data.Type))+geom_boxplot(stat='boxplot')+xlab('')+ylab('Gene length (bp)')+scale_fill_brewer(palette='Set2')+theme(panel.grid.minor=element_blank(), panel.grid.major=element_blank(),legend.position='none')+theme(axis.title.y = element_text(size=12, angle=90, hjust=0.5, vjust=0.25),axis.text.x=element_text(colour='black'))
vplayout <- function(x, y) viewport(layout.pos.row = x, layout.pos.col = y)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3, 2)))
print(p1,vp = vplayout(2, 1))
print(p2,vp = vplayout(1:3, 2))
dev.off();
";
	close R;
}
###################################
#### ANALYSIS OF GENE ONTOLOGY #### 
###################################

sub CountValuesPerKey {
	my ($Genome, $out) = @_;
	my %ProfileRes;
	unlink $out;
	open (OUT, '>>', $out);
	foreach my $key (keys %{$Genome}) {
		my %Name_hash;
		my $count = 0;
		for my $key1 (keys %{$Genome->{$key}->{'Genes'}}) {
			my $PID = $Genome->{$key}->{'Genes'}->{$key1}->{'PID'};
			$Name_hash{$PID}->{'Function'}=$Genome->{$key}->{'Genes'}->{$key1}->{'Function'};
			$Name_hash{$PID}->{'Pathway'}=$Genome->{$key}->{'Genes'}->{$key1}->{'Pathway'};
			$Name_hash{$PID}->{'Genes'}->{$key1}->{'Profile'}=$key;
		}
		my @Array = keys(%{$Genome->{$key}->{'Genes'}});
		my $random_key = $Array[int(rand(@Array))];
		$ProfileRes{$key}->{'Num'}=keys(%Name_hash);
		$ProfileRes{$key}->{'Function'}=$Genome->{$key}->{'Genes'}->{$random_key}->{'Function'};
		$ProfileRes{$key}->{'Pathway'}=$Genome->{$key}->{'Genes'}->{$random_key}->{'Pathway'};
		print OUT $key."\t".keys(%Name_hash)."\t".$Genome->{$key}->{'Genes'}->{$random_key}->{'Function'}."\t".$Genome->{$key}->{'Genes'}->{$random_key}->{'Pathway'}."\n";
	}
	close OUT;
	return %ProfileRes;
}	

sub Pathways2list {
	my ($File) = @_;
	my %Pathways;
	foreach my $key (keys %{$File}) {
		my @row = split('\|\|',$File->{$key}->{'Pathway'});
		foreach my $item (@row) {
			my @pws = split('\|', $item);
			foreach my $item1 (@pws) {
				if ($item1 =~ m/.*(GO:\d+)\_*.*/) {
					$Pathways{'GO'}->{$1}->{'Profiles'}->{$key}->{'Num'}=$File->{$key}->{'Num'};
					$Pathways{'GO'}->{$1}->{'Profiles'}->{$key}->{'Function'}=$File->{$key}->{'Function'};
					$Pathways{'GO'}->{$1}->{'Pathway'}=$item1;
				} elsif ($item1 =~ m/.*(KEGG:_\d+)\_*.*/) {
					$Pathways{'KEGG'}->{$1}->{'Profiles'}->{$key}->{'Num'}=$File->{$key}->{'Num'};
					$Pathways{'KEGG'}->{$1}->{'Profiles'}->{$key}->{'Function'}=$File->{$key}->{'Function'};
					$Pathways{'KEGG'}->{$1}->{'Pathway'}=$item1;
				}
			}
		}
	}
	return %Pathways;
}


sub PrintPWlist {
	my ($Pathways, $out) = @_;
	unlink $out;
	open (OUT, '>>', $out);
	#print OUT "DB_term\n";
	if ($pathwayDB eq 'GO') {
		foreach my $key (keys %{$Pathways->{'GO'}}) {
			print OUT "empty\tempty\tempty\t".$key."\t".$key."\n";
		
		}	
	} elsif ($pathwayDB eq 'KEGG') {
		foreach my $key (keys %{$Pathways->{'KEGG'}}) {
			print OUT $key."\t".keys(%{$Pathways->{'KEGG'}->{$key}->{'Profiles'}})."\t".$Pathways->{'KEGG'}->{$key}->{'Pathway'}."\n";
		}
	}
}

sub Map2SlimParser {
	my @TBL_CF = OpenFile(@_);
	my %Res;
	for (my $i=0;$i<=$#TBL_CF;$i++) {
		my @row = split('\t', $TBL_CF[$i]);
		$Res{$row[4]}->{'Initial GO IDs'}->{$row[3]}=$row[3];	
	}
	return %Res;
}


sub GO2Terms {
	my ($tbl_in, $tbl_out, $tmp_in, $R_path) = @_;
	unlink $tbl_out;
	open (OUT, '>>', $tbl_out);
	unlink $tmp_in;
	my %GO2SlimRes;
	system "map2slim $scripts_path/go-perl/goslim_generic.obo.obo $scripts_path/go-perl/gene_ontology.1_2.obo.obo $tbl_in > $tmp_in";
	my $parser = new GO::Parser({handler=>'obj'});
	my $ggpath = $scripts_path."/go-perl/goslim_generic.obo.obo";
	$parser->parse($ggpath);
	my $graph = $parser->handler->graph;
  	my %GO2Slim = Map2SlimParser($tmp_in);
  	print OUT "GO:ID\tGO2Slim_ancestor\tONTOLOGY\tTERM\n";
  	foreach my $key (keys %GO2Slim) {
  		my $term = $graph->get_term("$key");
  		$GO2SlimRes{$term->namespace}->{'Common terms'}->{$term->name}->{'Initial GO IDs'}=$GO2Slim{$key}->{'Initial GO IDs'};
  		$GO2SlimRes{$term->namespace}->{'Count of GO IDs'} += keys(%{$GO2Slim{$key}->{'Initial GO IDs'}});
  		foreach my $key1 (keys %{$GO2Slim{$key}->{'Initial GO IDs'}}) {
  			print OUT $key1."\t".$key."\t".$term->namespace."\t".$term->name."\n";
  		}
  	}
  	return %GO2SlimRes;
}

sub GO2Term_plot_R2 {
	my ($Go2Slim, $tbl_out, $plot, $R_path) = @_;
	unlink $tbl_out;
	open (OUT, '>>', $tbl_out);
	print OUT "ONTOLOGY\tTERM\tFRACTION\tNUMBER\n";
	foreach my $key (sort keys %{$Go2Slim}) {
		foreach my $key1 (keys %{$Go2Slim->{$key}->{'Common terms'}}) {
			my $fraction = (keys(%{$Go2Slim->{$key}->{'Common terms'}->{$key1}->{'Initial GO IDs'}})/$Go2Slim->{$key}->{'Count of GO IDs'})*100;
			
			print OUT $key."\t".$key1."\t".$fraction."\t".keys(%{$Go2Slim->{$key}->{'Common terms'}->{$key1}->{'Initial GO IDs'}})."\n";
		}
	}
	open R , "| $R_path --vanilla > /dev/null";
	print R "
postscript('$plot',horizontal=TRUE)
library(ggplot2)
data <- read.table('$tbl_out', header=T, sep='\t');
data\$TERM = as.character(data\$TERM)
data\$TERM <- factor(data\$TERM, levels=unique(data\$TERM), ordered=TRUE)
BP <- subset(data, ONTOLOGY == 'biological_process');
CC <- subset(data, ONTOLOGY == 'cellular_component');
MF <- subset(data, ONTOLOGY == 'molecular_function');
pie_df= data.frame(count = c(sum(BP\$NUMBER), sum(CC\$NUMBER), sum(MF\$NUMBER)), Type = c('biological process', 'cellular component', 'molecular function'))
pie_df\$fraction = (pie_df\$count)/ sum(pie_df\$count)
pie_df\$fraction = pie_df\$fraction*100
p1 = ggplot(data=pie_df, aes(x=Type, y=fraction, fill=Type))+geom_bar(stat='identity')+xlab('')+ylab('Fraction of GO IDs (%)')+geom_text(aes(label=count), colour='black', size=3, vjust=1, hjust=1.1)+opts(panel.grid.minor=theme_blank(), panel.grid.major=theme_blank(),legend.position='none',panel.background=theme_rect(fill='transparent'),axis.title.y = theme_text(size=8, angle=00, hjust=0.75, vjust=0))+scale_fill_brewer(palette='Set1')+scale_fill_discrete(name='')+coord_flip()+opts(axis.title.x = theme_text(size=9, angle=00, hjust=1, vjust=0),axis.text.y=theme_text(colour='black'))
p2 <- ggplot(data=data, aes(x=TERM, y=FRACTION, fill=ONTOLOGY))+geom_bar(stat='identity', bandwidth=1)+xlab('')+ylab('Fraction of GO IDs (%)')+geom_text(aes(label=NUMBER), colour='black', size=2.5, vjust=0)+opts(panel.grid.minor=theme_blank(),legend.position='none',panel.grid.major=theme_blank(),panel.background=theme_rect(fill='transparent'))+scale_fill_brewer(palette='Set1')+scale_fill_discrete(name='')+opts(axis.title.y = theme_text(size=9, angle=90, hjust=0.75, vjust=0.25),axis.text.x=theme_text(angle=90,hjust=1,vjust=0.5,size=7))
vplayout <- function(x, y) viewport(layout.pos.row = x, layout.pos.col = y)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3, 8)))
print(p1,vp = vplayout(2, 1:2))
print(p2,vp = vplayout(1:3, 3:8))
dev.off();
";
	close R;
}

sub GO2Term_plot_R3 {
	my ($Go2Slim, $tbl_out, $plot, $R_path) = @_;
	unlink $tbl_out;
	open (OUT, '>>', $tbl_out);
	print OUT "ONTOLOGY\tTERM\tFRACTION\tNUMBER\n";
	foreach my $key (sort keys %{$Go2Slim}) {
		foreach my $key1 (keys %{$Go2Slim->{$key}->{'Common terms'}}) {
			my $fraction = (keys(%{$Go2Slim->{$key}->{'Common terms'}->{$key1}->{'Initial GO IDs'}})/$Go2Slim->{$key}->{'Count of GO IDs'})*100;
			
			print OUT $key."\t".$key1."\t".$fraction."\t".keys(%{$Go2Slim->{$key}->{'Common terms'}->{$key1}->{'Initial GO IDs'}})."\n";
		}
	}
	open R , "| $R_path --vanilla > /dev/null";
	print R "
postscript('$plot',horizontal=TRUE)
library(ggplot2)
library(grid)
data <- read.table('$tbl_out', header=T, sep='\t');
data\$TERM = as.character(data\$TERM)
data\$TERM <- factor(data\$TERM, levels=unique(data\$TERM), ordered=TRUE)
BP <- subset(data, ONTOLOGY == 'biological_process');
CC <- subset(data, ONTOLOGY == 'cellular_component');
MF <- subset(data, ONTOLOGY == 'molecular_function');
pie_df= data.frame(count = c(sum(BP\$NUMBER), sum(CC\$NUMBER), sum(MF\$NUMBER)), Type = c('biological process', 'cellular component', 'molecular function'))
pie_df\$fraction = (pie_df\$count)/ sum(pie_df\$count)
pie_df\$fraction = pie_df\$fraction*100
p1 = ggplot(data=pie_df, aes(x=Type, y=fraction, fill=Type))+geom_bar(stat='identity')+xlab('')+ylab('Fraction of GO IDs (%)')+geom_text(aes(label=count), colour='black', size=3, vjust=1, hjust=1.1)+theme(panel.grid.minor=element_blank(), panel.grid.major=element_blank(),legend.position='none',axis.title.y = element_text(size=8, angle=00, hjust=0.75, vjust=0))+scale_fill_brewer(palette='Set2')+coord_flip()+theme(axis.title.x = element_text(size=9, angle=00, hjust=1, vjust=0),axis.text.y=element_text(colour='black'))
p2 <- ggplot(data=data, aes(x=TERM, y=FRACTION, fill=ONTOLOGY))+geom_bar(stat='identity', bandwidth=1)+xlab('')+ylab('Fraction of GO IDs (%)')+geom_text(aes(label=NUMBER), colour='black', size=2.5, vjust=0)+theme(panel.grid.minor=element_blank(),legend.position='none',panel.grid.major=element_blank())+scale_fill_brewer(palette='Set2')+theme(axis.title.y = element_text(size=9, angle=90, hjust=0.75, vjust=0.25),axis.text.x=element_text(angle=90,hjust=1,vjust=0.5,size=7))
vplayout <- function(x, y) viewport(layout.pos.row = x, layout.pos.col = y)
grid.newpage()
pushViewport(viewport(layout = grid.layout(3, 8)))
print(p1,vp = vplayout(2, 1:2))
print(p2,vp = vplayout(1:3, 3:8))
dev.off();
";
	close R;
}

sub OpenFile {
	my ($Input) = @_;
	my @Filein;
	open (IN, '<', $Input) or die "Can't read file $Input: $!";
	while (<IN>) {
		next if (/^(\s)*$/) or (/^#/);
		chomp;
		push (@Filein, $_);
	}
	close IN;
	return @Filein;
}

###########################################
#### PAIRWISE SPECIFIC GENOME ANALYSIS #### 
###########################################

sub SpecificMatrix {
	my ($list_in) = @_;
	my %Res;
	my @Array=@{$list_in};
	for (my $i=0;$i<=$#Array;$i++) {
		my $item1 = $Array[$i];
		my ($f1, $d1) = fileparse($item1);
		$f1 =~ s/\.\w+\.\d+\.profiles$//;
		my %QueryProfiles = OpenProfiles_PW($item1);
		for (my $j=0;$j<=$#Array;$j++) {
			my $item2 = $Array[$j];
			my ($f2, $d2) = fileparse($item2);
			$f2 =~ s/\.\w+\.\d+\.profiles$//;
			my %SubjectProfiles = OpenProfiles_PW($item2);
			my %Specific = CountSpecific(\%QueryProfiles, \%SubjectProfiles);
			my $fraction = keys(%Specific)/keys(%QueryProfiles);
			my $fraction_rn = sprintf("%.1f", $fraction*100);
			$Res{$i}->{'Genomes'}->{$j}->{'Fraction'}=$fraction_rn;
			$Res{$i}->{'Name'}=$f1;
			$Res{$i}->{"Genomes"}->{$j}->{'Number'}=keys(%Specific);
			$Res{$i}->{"Genomes"}->{$j}->{'Name'}=$f2;
		}
	}
	return %Res;
}

sub CountSpecific {
	my ($Hash1, $Hash2) = @_;
	my %Spec_hash;
	foreach my $key (keys %{$Hash1}) {
		if (! exists $Hash2->{$key}) {
			$Spec_hash{$key}=$Hash1->{$key};
		}
	}
	return %Spec_hash;
}

sub SpecificMatrix_plot_R2 {
	my ($list_in, $tbl_out, $plot, $R_path) = @_;
	my %Res = SpecificMatrix($list_in);
	unlink $tbl_out;
	open (TBLOUT, ">", $tbl_out);
	print TBLOUT "GENOME1\tGENOME2\tFraction\tNumber\n";
	foreach my $key (sort {$a <=> $b} keys %Res) {
		foreach my $key1 (sort {$a <=> $b} keys %{$Res{$key}->{'Genomes'}}) {
			print TBLOUT $Res{$key}->{'Name'}."\t".$Res{$key}->{'Genomes'}->{$key1}->{'Name'}."\t".$Res{$key}->{'Genomes'}->{$key1}->{'Fraction'}."\t".$Res{$key}->{'Genomes'}->{$key1}->{'Number'}."\n";
		}
	}
		open R , "| $R_path --vanilla > /dev/null";
	print R "
postscript('$plot');
library(ggplot2)
data <- read.table('$tbl_out', header=T, sep='\t');
data\$GENOME1 = as.character(data\$GENOME1);
data\$GENOME2 = as.character(data\$GENOME2);
midpoint=max(data\$Fraction)/2;
ggplot(data, aes(GENOME2,GENOME1,fill=Fraction,label=sprintf('%.1f%%',Fraction)))+geom_tile()+geom_text()+scale_fill_gradient2(low = 'white', mid='light green', high = 'dark green',midpoint=midpoint)+xlab('')+ylab('')+opts(axis.text.x=theme_text(angle=90, colour='black'),axis.text.y=theme_text(colour='black'))
dev.off();
";
	close R;
}

sub SpecificMatrix_plot_R3 {
	my ($list_in, $tbl_out, $plot, $R_path) = @_;
	my %Res = SpecificMatrix($list_in);
	unlink $tbl_out;
	open (TBLOUT, ">", $tbl_out);
	print TBLOUT "GENOME1\tGENOME2\tFraction\tNumber\n";
	foreach my $key (sort {$a <=> $b} keys %Res) {
		foreach my $key1 (sort {$a <=> $b} keys %{$Res{$key}->{'Genomes'}}) {
			print TBLOUT $Res{$key}->{'Name'}."\t".$Res{$key}->{'Genomes'}->{$key1}->{'Name'}."\t".$Res{$key}->{'Genomes'}->{$key1}->{'Fraction'}."\t".$Res{$key}->{'Genomes'}->{$key1}->{'Number'}."\n";
		}
	}
		open R , "| $R_path --vanilla > /dev/null";
	print R "
postscript('$plot');
library(ggplot2)
data <- read.table('$tbl_out', header=T, sep='\t');
data\$GENOME1 = as.character(data\$GENOME1);
data\$GENOME2 = as.character(data\$GENOME2);
midpoint=max(data\$Fraction)/2;
ggplot(data, aes(GENOME2,GENOME1,fill=Fraction,label=sprintf('%.1f%%',Fraction)))+geom_tile()+geom_text()+scale_fill_gradient2(low = 'white', mid='light green', high = 'dark green',midpoint=midpoint)+xlab('')+ylab('')+theme(axis.text.x=element_text(angle=90, colour='black'),axis.text.y=element_text(colour='black'))
dev.off();
";
	close R;
}



# --------------------------------------------------------------------
# %% Help Page/Documentation %%
#

sub print_help {
  print <<END;

NAME
	PanFunPro2apply - uses profile information, achieved by PanFunPro2profiles.pl script for pan-genome analysis
	Results of each analysis are displayed in both tab separated table and postscript file
   

SYNOPSIS
    perl PanFunPro2apply.pl [-keep] [-root] [-analysis] [-specific_type] [-set1] [-set2] [-OutPath] [-resultvisuals] [-h] 

DESCRIPTION
	PanFunPro2apply 
    	
    
OPTIONS
	       
   	-keep <Dirname>
   		Specify the name of the directory where proteome profiles are kept. PanFunPro2profiles uses PanFunPro as default.
   		 		
   	-analysis <AnalysisName>
   		Specify the type of analysis: 'core', 'pan', 'pancoreplot', 'pancore_matrix', 'specific_matrix', 'specific'.
   		
   		Options 'core', 'pan', 'pancoreplot', 'pancore_matrix', 'specific_matrix' require one set of genome, specified by set1 option. 
   		Option 'specific' requires two sets of genomes, specified by set1 and set2 options.
   		
   		Core, pan and specific results are stored in the OutPath directory in PanFunPro.profiles.fsa file
   		
   		Pancoreplot results are stored in the OutPath directory as postscript file and table with the following names: 
   		PanFunPro.PanCorePlotTable.txt and PanFunPro.PanCorePlot.ps 
   		
   		Pancore_matrix results are stored in the OutPath directory as postscript file and table with the following names: 
   		PanFunPro.PanCoreMatrixTable.txt and PanFunPro.PanCoreMatrix.ps
   		
   		Specific_matrix results are stored in the OutPath directory as postscript file and table with the following names: 
   		PanFunPro.SpecificMatrixTable.txt and PanFunPro.SpecificMatrix.ps
   		
   	-specific_type <Type>
   		Specific analysis type can be one of the following: i2c, i2ci, u2c, u2ci
   		i2c - Specific gene families are calculated as the core between one set of genomes which is complimentary to a pan of another set of genomes
   		i2ci - Specific gene families are calculated as the core between one set of genomes which is complimentary to a core of another set of genomes
   		u2c - Specific gene families are calculated as the pan between one set of genomes which is complimentary to a pan of another set of genomes
   		u2ci - Specific gene families are calculated as the pan between one set of genomes which is complimentary to a core of another set of genomes
   	
   	-root <Path>
   		Specify the Path of the directory, which will be set as current working directory. Default is set by getcwd() command
   		
   	-OutPath <Directoryname>
   		Specify the name of the directory, where results are going to be stored. `default is 'Output'
   		
	-set1 <Numbers>
		Specify the numbers of the genomes, that should be included in the analysis. PanFunPro2profiles.pl stores genomes in the list according to
		the number of the genome in the list. Example is following:
		
		PRJNA57623.proteins.fsa.1.profiles
		PRJNA57625.proteins.fsa.3.profiles
		
		Number 1 before 'profiles' means that genome is the first in the list, and number 3 before 'profiles' - genome is the third
		in the list. Numbers can be checked in the 'CorePanResults.txt' file in the column, by adding +1 to the id number. 
		
		Example is:
		
		-set1 1,2,3 or -set1 1:3 or -set1 1,2,4:6
		
	-set2 <Numbers>
		Specify the numbers of the genomes, that should be included in the analysis. PanFunPro2profiles.pl stores genomes in the list according to
		the number of the genome in the list. Example is following:
		
		PRJNA57623.proteins.fsa.1.profiles
		PRJNA57625.proteins.fsa.3.profiles
		
		Number 1 before 'profiles' means that genome is the first in the list, and number 3 before 'profiles' - genome is the third
		in the list. Numbers can be checked in the 'CorePanResults.txt' file in the column, by adding +1 to the id number. 
		
		Example is:
		
		-set2 1,2,3 or -set2 1:3 or -set2 1,2,4:6
    	
	-resultvisuals <Type>
		Core, pan, and specific results can include additional analysis: 'statistics' or 'GO'. Default is 'none'.
    	
		'Statistics' option shows the distribution of the the output profiles among PfamA, TIGRFAM, SuperFamily and CD-HIT. 
		Additionall gene length distribution in each group is visualized. Results are saved in the OutPath directory with as postscript file and table with
		the following names: PanFunPro.lengthstable.txt and PanFunPro.lengthsplot.ps
    	
		'GO' option shows distribution of the possible Gene Ontology (GO) information for the output profiles. Results are saved in the OutPath directory 
		with as postscript file and table with the following names: PanFunPro.GOTerms2Count.txt and PanFunPro.GOTerms2Count.ps
 
	-scripts_path <FullPath>
		Specify the Path of the directory, where go-perl directory is located. For example: -scripts_path /Users/oksana/Scripts
		
	-R_version <Number>
		Specify the version of the R, which can be either '2' or '3'. Default is '3'.
		
	-profilesoutput <Type>
		Specify the type of the resulting gene families output: 'single' or 'all'. Default is 'single'.
		
		'single' means that genes, reprsenting gene families will be outputed from the random genome. 
		
		'all' means that genes, reprsenting gene families will be outputed from the all analysed genomes.
    	
	-h|help
		Prints out description of the code
 
VERSION
    This version (1.0, August 11, 2013) is the first version of the script.

EXAMPLE
	Core genome analysis
		perl PanFunPro2apply.pl -analysis core -set1 1,3,4:6 -resultvisuals GO -keep PanFunPro -scripts_path /Users/oksana/Scripts
	Specific genome analysis
		perl PanFunPro2apply.pl -analysis specific -specific_type u2c -set1 4:6 -set2 1:3 -resultvisuals Statistics -keep PanFunPro -scripts_path /Users/oksana/Scripts
	Pancoreplot
		perl PanFunPro2apply.pl -analysis pancoreplot -set1 1,3,4:6 -keep PanFunPro
	Pancore_matrix
		perl PanFunPro2apply.pl -analysis pancore_matrix -set1 1,3,4:6 -keep PanFunPro
	Specific_matrix
		perl PanFunPro2apply.pl -analysis specific_matrix -set1 1,3,4:6 -keep PanFunPro

AUTHOR
    Oksana Lukjancenko, oksana\@cbs.dtu.dk

END
  exit;
}

